/*****************************************************************************
 * PROJECT: IPC (Interprocess Communication) Package
 *
 * (c) Copyright 2011 Reid Simmons.  All rights reserved.
 *
 * FILE:	IPC.i
 *
 * DESCRIPTION: SWIG interface file for Lisp interface
 *              Liberally adapted by Reid from initial work by 
 *              Trey Smith and Nathaniel Fairfield
 *
 *       $Id: IPC.i,v 2.1 2013/07/31 19:54:49 reids Exp $
 * $Revision: 2.1 $
 *     $Date: 2013/07/31 19:54:49 $
 *   $Author: reids $
 *    $State: Exp $
 *   $Locker:  $
 *
 * Copyright (c) 2011, Carnegie Mellon University
 *     This software is distributed under the terms of the 
 *     Simplified BSD License (see ipc/LICENSE.TXT)
 *
 * REVISION HISTORY
 * $Log: IPC.i,v $
 * Revision 2.1  2013/07/31 19:54:49  reids
 * Updated for using SWIG
 *
 *
 ****************************************************************/	

/**********************************************************************
 * INCLUDES
 **********************************************************************/

%module IPC

%{
// The code here will appear in the wrapper generated by SWIG.  The
// wrapper creates C functions that accept Lisp objects as arguments,
// translate those arguments as necessary, and then call the underlying
// C functions.  These directives need to be in the wrapper file so that
// the underlying C functions are available to the wrapper functions.

#include "ipc.h"
extern void SWIG_init(void); // To prevent compiler from complaining

#define NMP_IPC
#define NEED_DEBUGGING
#include "ipcFFI.c"
#include "ipcLisp.c"

%}

/**********************************************************************
 * MAIN BODY
 **********************************************************************/

# Functions that are rewritten for the Lisp version of IPC (see ipcLisp.lisp)
%rename(_IPC_connect) IPC_connect;
%rename(_IPC_connectModule) IPC_connectModule;
%rename(_IPC_connectNoListen) IPC_connectNoListen;
%rename(_IPC_connectModuleNoListen) IPC_connectModuleNoListen;
%rename(_IPC_isConnected) IPC_isConnected;
%rename(_IPC_isModuleConnected) IPC_isModuleConnected;
%rename(_IPC_isMsgDefined) IPC_isMsgDefined;
%rename(_IPC_initialize) IPC_initialize;
%rename(_IPC_parseFormat) IPC_parseFormat;
%rename(_IPC_removeTimerByRef) IPC_removeTimerByRef;
%rename(formatChoosePtrFormatC) formatChoosePtrFormat;
%rename(_IPC_defineMsg) IPC_defineMsg;

%ignore IPC_WAIT_FOREVER;
%ignore IPC_VARIABLE_LENGTH;
%ignore IPC_FIXED_LENGTH;
%ignore TRIGGER_FOREVER;

%ignore IPC_subscribe;
%ignore IPC_subscribeData;
%ignore IPC_unsubscribe;
%ignore IPC_subscribeFD;
%ignore IPC_unsubscribeFD;
%ignore IPC_subscribeConnect;
%ignore IPC_subscribeDisconnect;
%ignore IPC_unsubscribeConnect;
%ignore IPC_unsubscribeDisconnect;
%ignore IPC_subscribeHandlerChange;
%ignore IPC_unsubscribeHandlerChange;
%ignore IPC_queryNotify;
%ignore IPC_queryResponse;
%ignore IPC_queryNotifyVC;
%ignore IPC_queryResponseVC;
%ignore IPC_marshall;
%ignore IPC_unmarshall;
%ignore IPC_unmarshallData;
%ignore IPC_freeData;
%ignore IPC_freeDataElements;
%ignore IPC_publishData;
%ignore IPC_respondData;
%ignore IPC_queryNotifyData;
%ignore IPC_queryResponseData;
%ignore IPC_printData;
%ignore IPC_readData;
%ignore IPC_addTimer;
%ignore IPC_addTimerGetRef;
%ignore IPC_addOneShotTimer;
%ignore IPC_addPeriodicTimer;
%ignore IPC_removeTimer;
%ignore x_ipcRegisterExitProc;

%include "ipc.h"

#define DEFINE_FOR_SWIG
#define NEED_DEBUGGING
%include ipcFFI.h
%include ipcLisp.h

%insert("lisp") {
(load (make-pathname :directory LIB_DIR :name "libipc_lisp.so") :foreign t)
(load (make-pathname :directory LISP_DIR :name "primFmttrs"))
(load (make-pathname :directory LISP_DIR :name "formatters"))
(load (make-pathname :directory LISP_DIR :name "ipcLisp"))
}
