/*****************************************************************************
 * PROJECT: IPC (Interprocess Communication) Package
 *
 * (c) Copyright 2011 Reid Simmons.  All rights reserved.
 *
 * FILE:	IPC.i
 *
 * DESCRIPTION: SWIG interface file for Python interface
 *              Liberally adapted by Reid from initial work by 
 *              Trey Smith and Nathaniel Fairfield
 *
 *       $Id: IPC.i,v 1.3 2013/07/23 21:12:24 reids Exp $
 * $Revision: 1.3 $
 *     $Date: 2013/07/23 21:12:24 $
 *   $Author: reids $
 *    $State: Exp $
 *   $Locker:  $
 *
 * Copyright (c) 2011, Carnegie Mellon University
 *     This software is distributed under the terms of the 
 *     Simplified BSD License (see ipc/LICENSE.TXT)
 *
 * REVISION HISTORY
 * $Log: IPC.i,v $
 * Revision 1.3  2013/07/23 21:12:24  reids
 * Minor editing and bug fix
 *
 * Revision 1.2  2012/02/27 16:55:46  reids
 * Fixed some problems with python and significantly improved transfer of arrays to/from python
 *
 * Revision 1.1  2011/08/16 16:00:36  reids
 * Adding Python interface to IPC
 *
 ****************************************************************/	

/**********************************************************************
 * INCLUDES
 **********************************************************************/

%module IPC

%{
// The code here will appear in the wrapper generated by SWIG.  The
// wrapper creates C functions that accept python objects as arguments,
// translate those arguments as necessary, and then call the underlying
// C functions.  These directives need to be in the wrapper file so that
// the underlying C functions are available to the wrapper functions.

#include "ipc.h"
extern void SWIG_init(void); // To prevent compiler from complaining

#define NMP_IPC
#define NEED_DEBUGGING
#include "ipcFFI.c"
#include "ipcPython.c"

%}

/**********************************************************************
 * MAIN BODY
 **********************************************************************/

// Handling exceptions that occur within handlers (or exit proc)
%exception {
  $action
  if (PyErr_Occurred() != NULL) SWIG_fail;
}

# Functions that are rewritten for the Python version of IPC (see ipcPython.py)
%ignore IPC_subscribe;
%ignore IPC_subscribeData;
%ignore IPC_unsubscribe;
%ignore IPC_subscribeFD;
%ignore IPC_unsubscribeFD;
%ignore IPC_subscribeConnect;
%ignore IPC_subscribeDisconnect;
%ignore IPC_unsubscribeConnect;
%ignore IPC_unsubscribeDisconnect;
%ignore IPC_subscribeHandlerChange;
%ignore IPC_unsubscribeHandlerChange;
%ignore IPC_queryNotify;
%ignore IPC_queryResponse;
%ignore IPC_queryNotifyVC;
%ignore IPC_queryResponseVC;
%ignore IPC_marshall;
%ignore IPC_unmarshall;
%ignore IPC_unmarshallData;
%ignore IPC_freeData;
%ignore IPC_freeDataElements;
%ignore IPC_publishData;
%ignore IPC_respondData;
%ignore IPC_queryNotifyData;
%ignore IPC_queryResponseData;
%ignore IPC_printData;
%ignore IPC_readData;
%ignore IPC_addTimer;
%ignore IPC_addTimerGetRef;
%ignore IPC_addOneShotTimer;
%ignore IPC_addPeriodicTimer;
%ignore IPC_removeTimer;
%ignore x_ipcRegisterExitProc;

%include "ipc.h"

%pythoncode {
import formatters
from formatters import IPCdata
from sys import exc_info
}

#define DEFINE_FOR_SWIG
#define NEED_DEBUGGING
%include ipcFFI.h
%include ipcPython.h

%include ipcPython.py
